@@ -115,7 +115,8 @@ static gint gnome_print_ps2_set_line (GnomePrintPs2 *ps2);
 static gint gnome_print_ps2_set_dash (GnomePrintPs2 *ps2);
 
 static gint gnome_print_ps2_set_color_real (GnomePrintPs2 *ps2, gdouble r, gdouble g, gdouble b);
-static gint gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *font);
+
+static gint gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font, gboolean subfont_flag, gint subfont_number);
 
 static gint gnome_print_ps2_print_bpath (GnomePrintPs2 *ps2, const ArtBpath *bpath);
 
@@ -248,8 +249,8 @@ gnome_print_ps2_construct (GnomePrintContext *ctx)
 	ret = gnome_print_transport_open (ctx->transport);
 	g_return_val_if_fail (ret >= 0, ret);
 
-	tmp = g_strdup ("/tmp/gnome-print-XXXXXX");
-	fd = mkstemp (tmp);
+	tmp = g_build_filename(g_get_tmp_dir (), "gnome-print-XXXXXX", NULL);
+	fd = g_mkstemp (tmp);
 	if (fd < 0) {
 		g_warning ("file %s: line %d: Cannot create temporary file", __FILE__, __LINE__);
 		g_free (tmp);
@@ -471,12 +472,17 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 
 	for (s = 0; s < pgl->num_strings; s++) {
 		GnomePosString * ps;
-		gint i;
+		GnomeFont *font;
+		gint i, current_page = 0; /* placate compiler */
 
 		ps = pgl->strings + s;
+		font = gnome_rfont_get_font (ps->rfont);
+
+		if (!NEEDED_SUBSETTING(font)) {
+			ret = gnome_print_ps2_set_font_real (ps2, font, FALSE, 0);
+			g_return_val_if_fail (ret >= 0, ret);
+		}
 
-		ret = gnome_print_ps2_set_font_real (ps2, gnome_rfont_get_font (ps->rfont));
-		g_return_val_if_fail (ret >= 0, ret);
 		ret = gnome_print_ps2_set_color_real (ps2,
 						((ps->color >> 24) & 0xff) / 255.0,
 						((ps->color >> 16) & 0xff) / 255.0,
@@ -493,9 +499,29 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 		g_return_val_if_fail (ret >= 0, ret);
 		/* Build string */
 
-		ret = gnome_print_ps2_fprintf (ps2, "(");
+		if (NEEDED_SUBSETTING(font)) {
+			for (i = ps->start; i < ps->start + ps->length; i++) {
+				gint master_glyph, glyph, page;
+                                master_glyph = pgl->glyphs[i].glyph; 
+                                page = master_glyph / 255;
 
-		if (ps2->selected_font->pso->encodedbytes == 1) {
+                                if (i==ps->start) {
+					current_page = page;
+					gnome_print_ps2_set_font_real (ps2, font, TRUE, page);
+					ret = gnome_print_ps2_fprintf (ps2, "(");
+				} else if (current_page != page) {
+                                        current_page = page;
+                                        (void)gnome_print_ps2_fprintf (ps2, ")show" EOL);
+					gnome_print_ps2_set_font_real (ps2, font, TRUE, page);
+					ret = gnome_print_ps2_fprintf (ps2, "(");
+				}
+				gnome_font_face_pso_mark_glyph (ps2->selected_font->pso, master_glyph);
+				glyph = master_glyph ? master_glyph % 255 + 1 : master_glyph;
+				ret = gnome_print_ps2_fprintf (ps2, "\\%o", glyph);
+				g_return_val_if_fail (ret >= 0, ret);
+			}	
+		} else if (ps2->selected_font->pso->encodedbytes == 1) {
+			ret = gnome_print_ps2_fprintf (ps2, "(");
 			/* 8-bit encoding */
 			for (i = ps->start; i < ps->start + ps->length; i++) {
 				gint glyph;
@@ -505,6 +531,7 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 				g_return_val_if_fail (ret >= 0, ret);
 			}
 		} else {
+			ret = gnome_print_ps2_fprintf (ps2, "(");
 			/* 16-bit encoding */
 			for (i = ps->start; i < ps->start + ps->length; i++) {
 				gint glyph, page;
@@ -515,27 +542,33 @@ gnome_print_ps2_glyphlist (GnomePrintContext *pc, const gdouble *a, GnomeGlyphLi
 				g_return_val_if_fail (ret >= 0, ret);
 			}
 		}
-		ret = gnome_print_ps2_fprintf (ps2, ")" EOL);
-		g_return_val_if_fail (ret >= 0, ret);
-		/* Build array */
-		ret = gnome_print_ps2_fprintf (ps2, "[");
-		g_return_val_if_fail (ret >= 0, ret);
-		for (i = ps->start + 1; i < ps->start + ps->length; i++) {
-			ret = gnome_print_ps2_print_double 
-				(ps2, "%g", 
-				 pgl->glyphs[i].x - pgl->glyphs[i-1].x);
-			ret += gnome_print_ps2_fprintf (ps2, " ");
-			ret += gnome_print_ps2_print_double
-				(ps2, "%g",
-				 pgl->glyphs[i].y - pgl->glyphs[i-1].y);
-			ret += gnome_print_ps2_fprintf (ps2, " ");
+
+		if (!NEEDED_SUBSETTING(font)) {
+			ret = gnome_print_ps2_fprintf (ps2, ")" EOL);
+			g_return_val_if_fail (ret >= 0, ret);
+			/* Build array */
+			ret = gnome_print_ps2_fprintf (ps2, "[");
+			g_return_val_if_fail (ret >= 0, ret);
+			for (i = ps->start + 1; i < ps->start + ps->length; i++) {
+				ret = gnome_print_ps2_print_double 
+					(ps2, "%g", 
+					 pgl->glyphs[i].x - pgl->glyphs[i-1].x);
+				ret += gnome_print_ps2_fprintf (ps2, " ");
+				ret += gnome_print_ps2_print_double
+					(ps2, "%g",
+					 pgl->glyphs[i].y - pgl->glyphs[i-1].y);
+				ret += gnome_print_ps2_fprintf (ps2, " ");
+				g_return_val_if_fail (ret >= 0, ret);
+			}
+			ret = gnome_print_ps2_fprintf (ps2, "0 0] ");
+			g_return_val_if_fail (ret >= 0, ret);
+			/* xyshow */
+			ret = gnome_print_ps2_fprintf (ps2, "xyshow" EOL);
+			g_return_val_if_fail (ret >= 0, ret);
+		} else {
+			ret = gnome_print_ps2_fprintf (ps2, ")show" EOL);
 			g_return_val_if_fail (ret >= 0, ret);
 		}
-		ret = gnome_print_ps2_fprintf (ps2, "0 0] ");
-		g_return_val_if_fail (ret >= 0, ret);
-		/* xyshow */
-		ret = gnome_print_ps2_fprintf (ps2, "xyshow" EOL);
-		g_return_val_if_fail (ret >= 0, ret);
 	}
 
 	if (!identity) {
@@ -994,15 +1027,40 @@ gnome_print_ps2_set_color_real (GnomePrintPs2 *ps2, gdouble r, gdouble g, gdoubl
 }
 
 static gint
-gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font)
+gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font, gboolean subfont_flag, gint subfont_number)
 {
 	const GnomeFontFace *face;
 	GnomePrintPs2Font *font;
-	gint ret = 0;
 	GSList *l;
 	gint instance = 0;
+	guchar *encodedstring = NULL; /* placate gcc */
+	gint ret = 0;
 
-	if ((ps2->selected_font) &&
+	face = gnome_font->face;
+
+	if (subfont_flag) {
+		guchar *current_str;
+                if (subfont_number == 0)
+                        encodedstring = g_strdup_printf ("(GnomeUni-%s) cvn FF %g",gnome_font->face->psname, gnome_font_get_size (gnome_font));
+                else
+                        encodedstring = g_strdup_printf ("(GnomeUni-%s_%03d) cvn FF %g",gnome_font->face->psname, subfont_number, gnome_font_get_size (gnome_font));
+
+		if (ps2->selected_font &&
+		    ps2->selected_font->pso &&
+		    ps2->selected_font->pso->encodedname) {
+
+			current_str = g_strdup_printf ("(%s) cvn FF %g", ps2->selected_font->pso->encodedname, ps2->selected_font->currentsize);
+
+			if (!strcmp (encodedstring, current_str)) {
+				g_free (encodedstring);
+				g_free (current_str);
+				return 0;
+			}
+		}
+        }
+
+
+	if (!subfont_flag && (ps2->selected_font) &&
 	    (ps2->selected_font->face == gnome_font->face) &&
 	    (ps2->selected_font->currentsize == gnome_font->size)) {
 		return 0;
@@ -1011,9 +1069,18 @@ gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font)
 	face = gnome_font_get_face (gnome_font);
 
 	for (font = ps2->fonts; font != NULL; font = font->next) {
-		if (font->face == face)
+		if (subfont_flag) {
+			gchar *newencodedstring = g_strdup_printf ("(%s) cvn FF %g",
+				font->pso->encodedname, font->currentsize);
+			if  (!strcmp (newencodedstring, encodedstring)) {
+				g_free (newencodedstring);
+				break;
+			}
+			g_free (newencodedstring);
+		} else if (font->face == face)
 			break;
 	}
+
 	if (!font) {
 		/* See #105063 */
 		GnomePrintPs2Font *clone;
@@ -1029,8 +1096,8 @@ gnome_print_ps2_set_font_real (GnomePrintPs2 *ps2, const GnomeFont *gnome_font)
 		font = g_new0 (GnomePrintPs2Font, 1);
 		font->next     = ps2->fonts;
 		font->face     = (GnomeFontFace *) face;
-		font->pso      = gnome_font_face_pso_new ((GnomeFontFace *) face, NULL, instance);	
-		font->instance = instance;
+		font->pso      = gnome_font_face_pso_new ((GnomeFontFace *) face, NULL, subfont_number);	
+		font->instance = subfont_number;
 
 		g_return_val_if_fail (font->pso != NULL, GNOME_PRINT_ERROR_UNKNOWN);
 
