@@ -33,7 +33,7 @@
  *
  */
 
-/* $Id: sft.h,v 1.1 2004/06/21 20:39:07 kcsuresh Exp $ */
+/* $Id: sft.h,v 1.3 2004/07/20 22:04:39 jody Exp $ */
 /* @(#)sft.h 1.17 03/01/08 SMI */
 
 /**
@@ -86,11 +86,14 @@
 #ifdef STSF
 #include <sttypes.h>
 #endif
+#include "config.h"
+#include "../gnome-print-private.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#if 0	/* Use glib's G_BYTE_ORDER == G_BIG_ENDIAN instead */
 #ifdef __sparc
 #ifndef G_BIG_ENDIAN
 #define G_BIG_ENDIAN
@@ -130,6 +133,7 @@ extern "C" {
 #if defined(G_BIG_ENDIAN) && defined(G_LITTLE_ENDIAN)
 #error "This is bizarre"
 #endif
+#endif
 
 #if 0  /* These should be defined in the makefile */
 #define DEBUG      /* Generate debugging output */
@@ -778,6 +782,7 @@ extern "C" {
 #ifdef USE_GSUB
         void   *pGSubstitution;                       /* info provided by GSUB for UseGSUB()                                */
 #endif
+	GnomePrintBuffer gp_buf;
     };
 
 #ifdef __cplusplus
