@@ -92,7 +92,7 @@ sub run_acroread
 	   print "Error while running acroread, missing argument pdf_file\n";
 	   return 1;
     }
-    
+
     $ret = &my_run_command ($command, $quiet);
 
     my $size = (stat ($test_result))[7];
@@ -298,7 +298,7 @@ sub my_check_gpa
 #
 sub my_check_output
 {
-    my $max = 6;
+    my $max = 7;
     my (@tests) = (
 		   ["ps",   \&run_gs],
 		   ["meta", \&run_meta], # Always run meta before acroread because
@@ -331,7 +331,7 @@ sub my_check_output
 		  if ($ret ne 0) {
 			 print "Error.. command failed\n";
 			 exit 1;
-		  } 
+		  }
 		  
 		  # Run the test on the file
 		  if (defined $proc) {
@@ -350,8 +350,8 @@ sub my_check_output
 
 
 #
-#
-#
+# Scan thru the list of fonts on the system and test the output generated
+# when embeding each font in a ps and pdf job
 #
 sub my_check_fonts
 {
@@ -377,7 +377,7 @@ sub my_check_fonts
 	   chomp ($font);
 
 	   print "  Testing font \#" . ($i + 1) . " [" . $font . "] on ps ... ";
-	   $output_file = "output/font_test" . ($i + 1) . ".ps";
+	   $output_file = sprintf ("output/font_test_%03d.ps", ($i + 1));
 	   $command =  "./fonts --generate=" . ($i + 1);
 	   $command .= " --output=" . $output_file . " > " . $test_result;
 
@@ -392,8 +392,7 @@ sub my_check_fonts
 		  print "pass\n";
 	   }
 
-	   print "  Testing font \#" . ($i + 1) . " [" . $font . "] on pdf acroread4... ";
-	   $output_file = "output/font_test" . ($i + 1) . ".pdf";
+	   $output_file = sprintf ("output/font_test_%03d.pdf", ($i + 1));
 	   $command =  "./fonts --generate=" . ($i + 1);
 	   $command .= " --output=" . $output_file . " --pdf > " . $test_result;
 	   $ret = &my_run_command ($command);
@@ -405,6 +404,16 @@ sub my_check_fonts
 		  exit -5;
 	   }
 	   
+	   print "  Testing font \#" . ($i + 1) . " [" . $font . "] on pdf ghostscript... ";
+	   $ret = run_gs ($output_file);
+	   if ($ret ne 0) {
+		  $failed++;
+		  print "FAILED\n";
+	   } else {
+		  print "pass\n";
+	   }
+
+	   print "  Testing font \#" . ($i + 1) . " [" . $font . "] on pdf acroread4... ";
 	   $ret = run_acroread ($output_file);
 	   if ($ret ne 0) {
 		  $failed++;
@@ -434,6 +443,73 @@ sub my_check_fonts
     
 }
 
+#
+# Check that the catalog generated is valid
+#
+sub my_check_catalog
+{
+    my $output_file;
+    my $command;
+    my $ret;
+    my $failed = 0;
+    
+    print "\nTesting catalog ...\n";
+
+    print "  Testing catalog on ps ... ";
+    $output_file = "output/catalog.ps";
+    $command =  "./fonts --catalog ";
+    $command .= "--output=" . $output_file . " > " . $test_result;
+
+    $ret = &my_run_command ($command);
+    if ($ret eq 0) {
+	   $ret = run_gs ($output_file, 0);
+    }
+    if ($ret ne 0) {
+	   $failed++;
+	   print "FAILED\n";
+    } else {
+	   print "pass\n";
+    }
+
+    print "  Testing catalog on pdf acroread4... ";
+    $output_file = "output/catalog.pdf";
+    $command =  "./fonts --catalog ";
+    $command .= "--output=" . $output_file . " --pdf > " . $test_result;
+    $ret = &my_run_command ($command);
+    if ($ret ne 0) {
+	   return $ret;
+    }
+    if ($ret eq -5) {
+	   # Crashed
+	   exit -5;
+    }
+	   
+    $ret = run_acroread ($output_file);
+    if ($ret ne 0) {
+	   $failed++;
+	   print "FAILED\n";
+    } else {
+	   print "pass\n";
+    }
+
+    print "  Testing catalog on pdf acroread5... ";
+    $ret = run_acroread5 ($output_file);
+    if ($ret ne 0) {
+	   $failed++;
+	   print "FAILED\n";
+    } else {
+	   print "pass\n";
+    }
+	   
+    if ($failed ne 0) {
+	   print "\n";
+	   print "Catalog test failed\n\n";
+ 	   print "run-test.pl unsucsessfull\n";
+	   exit 1;
+    }
+    
+}
+
 # Check that we can compile a gnome-print app, in this case we
 # test generate.c
 sub my_compile
@@ -631,7 +707,7 @@ sub my_usage
     print "Usage: ./run-test.pl [tests]\n";
     print "\n";
     print "Where test can be any set of:\n";
-    print "\tall check gpa errors output fonts\n";
+    print "\tall check gpa errors output catalog fonts\n";
     print "\n";
     print "Test \"all\" is ran if no tests are specified on the command line\n";
     print "\n";
@@ -645,12 +721,19 @@ sub my_check_all
     &my_check_gpa ();
     &my_check_errors ();
     &my_check_output ();
+    &my_check_catalog ();
     &my_check_fonts ();
 }
 
+sub my_clean
+{
+    system("rm output/*.ps output/*.pdf output/*.result output/*.error > /dev/null 2>&1");
+}
 
 my $arg;
 
+&my_clean ();
+
 if (scalar (@ARGV) eq 0) {
     &my_check_all ();
 } else {
@@ -670,6 +753,9 @@ if (scalar (@ARGV) eq 0) {
 	   if ($arg eq "output") {
 		  &my_check_output ();
 	   }
+	   if ($arg eq "catalog") {
+		  &my_check_catalog ();
+	   }
 	   if ($arg eq "fonts") {
 		  &my_check_fonts ();
 	   }
